"""VROOM input definition."""
from __future__ import annotations
from typing import Dict, Optional, Sequence, Union
from pathlib import Path

from numpy.typing import ArrayLike
import numpy

from .. import _vroom

from ..solution.solution import Solution
from ..job import Job
from ..vehicle import Vehicle


class Input(_vroom.Input):
    """VROOM input defintion.

    Main instance for adding jobs, shipments, vehicles, and cost and duration
    matrice defining a routing problem. Duration matrices is if not provided
    can also be retrieved from a map server.

    Attributes:
        jobs:
            Jobs that needs to be completed in the routing problem.
        vehicles:
            Vehicles available to solve the routing problem.

    """

    def __init__(
        self,
        amount_size: Optional[int] = None,
        servers: Optional[Dict[str, Union[str, _vroom.Server]]] = None,
        router: _vroom.ROUTER = _vroom.ROUTER.OSRM,
    ) -> None:
        """Class initializer.

        Args:
            amount_size:
                The size of the job to be transported. Used to verify all jobs
                have the same size limit.
            servers:
                Assuming no custom duration matrix is provided (from
                `set_durations_matrix`), use coordinates and a map server to
                calculate durations matrix. Keys should be identifed by
                `add_routing_wrapper`. If string, values should be on the
                format `{host}:{port}`.
            router:
                If servers is used, define what kind of server is provided.
                See `vroom.ROUTER` enum for options.
        """
        if servers is None:
            servers = {}
        for key, server in servers.items():
            if isinstance(server, str):
                servers[key] = _vroom.Server(*server.split(":"))
        self._amount_size = amount_size
        self._servers = servers
        self._router = router
        _vroom.Input.__init__(self, servers=servers, router=router)
        if amount_size is not None:
            self._set_amount_size(amount_size)

    def __repr__(self) -> str:
        """String representation."""
        args = []
        if self._amount_size is not None:
            args.append(f"amount_size={self._amount_size}")
        if self._servers:
            args.append(f"servers={self._servers}")
        if self._router != _vroom.ROUTER.OSRM:
            args.append(f"router={self._router}")
        return f"{self.__class__.__name__}({', '.join(args)})"

    @staticmethod
    def from_json(
        filepath: Path,
        servers: Optional[Dict[str, Union[str, _vroom.Server]]] = None,
        router: _vroom.ROUTER = _vroom.ROUTER.OSRM,
        geometry: Optional[bool] = None,
    ) -> Input:
        """Load model from JSON file.

        Args:
            filepath:
                Path to JSON file with problem definition.
            servers:
                Assuming no custom duration matrix is provided (from
                `set_durations_matrix`), use coordinates and a map server to
                calculate durations matrix. Keys should be identifed by
                `add_routing_wrapper`. If string, values should be on the
                format `{host}:{port}`.
            router:
                If servers is used, define what kind of server is provided.
                See `vroom.ROUTER` enum for options.
            geometry:
                Use coordinates from server instead of from distance matrix.
                If omitted, defaults to `servers is not None`.

        Returns:
            Input instance with all jobs, shipments, etc. added from JSON.

        """
        geometry = servers is not None
        instance = Input(servers=servers, router=router)
        with open(filepath) as handle:
            instance._from_json(handle.read(), geometry)
        return instance

    def set_geometry(self):
        return self._set_geometry()

    def add_job(
        self,
        job: Union[Job, Sequence[Job]],
    ) -> None:
        if isinstance(job, _vroom.Job):
            job = [job]
        for job_ in job:
            self._add_job(job_)

    def add_shipment(
        self,
        pickup: Job,
        delivery: Job,
    ) -> None:
        self._add_shipment(pickup, delivery)

    def add_vehicle(
        self,
        vehicle: Union[Vehicle, Sequence[Vehicle]],
    ) -> None:
        """Add vehicle.

        Args:
            vehicle:
                Vehicles to use to solve the vehicle problem. Vehicle type must
                have a recognized profile, and all added vehicle must have the
                same capacity.
        """
        if isinstance(vehicle, _vroom.Vehicle):
            vehicle = [vehicle]
        if not vehicle:
            return
        amount_sizes = {len(vehicle_.capacity) for vehicle_ in vehicle}
        if self._amount_size is not None:
            amount_sizes.add(self._amount_size)
        if len(amount_sizes) > 1:
            raise _vroom.VroomInputException(
                f"Incosistent capacity lengths: {amount_sizes}")
        if self._amount_size is None:
            size = amount_sizes.pop()
            self._amount_size = size
            self._set_amount_size(size)
        for vehicle_ in vehicle:
            self._add_vehicle(vehicle_)

    def set_durations_matrix(
        self,
        profile: str,
        matrix_input: ArrayLike,
    ) -> None:
        """Set durations matrix.

        Args:
            profile:
                Name of the transportation category profile in question.
                Typically "car", "truck", etc.
            matrix_input:
                A square matrix consisting of duration between each location of
                interest. Diagonal is canonically set to 0.
        """
        assert isinstance(profile, str)
        if not isinstance(matrix_input, _vroom.Matrix):
            matrix_input = _vroom.Matrix(numpy.asarray(matrix_input, dtype="uint32"))
        self._set_durations_matrix(profile, matrix_input)

    def set_costs_matrix(
        self,
        profile: str,
        matrix_input: ArrayLike,
    ) -> None:
        """Set costs matrix.

        Args:
            profile:
                Name of the transportation category profile in question.
                Typically "car", "truck", etc.
            matrix_input:
                A square matrix consisting of duration between each location of
                interest. Diagonal is canonically set to 0.
        """
        assert isinstance(profile, str)
        if not isinstance(matrix_input, _vroom.Matrix):
            matrix_input = _vroom.Matrix(numpy.asarray(matrix_input, dtype="uint32"))
        self._set_costs_matrix(profile, matrix_input)

    def solve(
        self,
        exploration_level: int,
        nb_threads: int,
    ) -> Solution:
        return Solution(self._solve(
            exploration_level=exploration_level,
            nb_threads=nb_threads,
        ))



from __future__ import annotations
from typing import Sequence, Tuple, Union

from . import _vroom


class LocationIndex(_vroom.Location):
    """Index in the custom duration matrix for where to find distances.

    Attributes:
        index:
            Location index referring to column in the duration
            matrix.

    Args:
        index:
            Location index referring to column in the duration
            matrix.
        location:
            Other location with `index` attribute to make a copy of.

    Examples:
        >>> loc = LocationIndex(4)
        >>> loc
        vroom.LocationIndex(4)
        >>> loc.index
        4

    See also:
        :cls:`vroom.Location`

    """

    def __init__(
        self,
        index: Union[int, Location],
    ) -> None:
        if isinstance(index, _vroom.Location):
            if not index._user_index():
                name = index.__class__.__name__
                raise TypeError(f"Can not convert {name} to LocationIndex")
            index = index._index()
        _vroom.Location.__init__(self, index)
        assert not self._has_coordinates()

    @property
    def index(self) -> int:
        """Location index."""
        return self._index()

    def __repr__(self) -> str:
        return f"vroom.{self.__class__.__name__}({self.index})"


class LocationCoordinates(_vroom.Location):
    """Location longitude and latitude.

    Attributes:
        index:
            Location index referring to column in the duration
            matrix.
        coords:
            Longitude and latitude coordinate.

    Args:
        coords:
            Longitude and latitude coordinate.
        location:
            Other location with `coords` attribute to make a copy of.

    Examples:
        >>> loc = LocationCoordinates([2., 3.])
        >>> loc
        vroom.LocationCoordinates((2.0, 3.0))
        >>> loc.coords
        (2.0, 3.0)

    See also:
        :cls:`vroom.Location`

    """

    def __init__(
        self,
        coords: Union[Location, Sequence[float]],
    ) -> None:
        if isinstance(coords, _vroom.Location):
            if not coords._has_coordinates():
                name = coords.__class__.__name__
                raise TypeError(f"Can not convert {name} to LocationCoordinates")
            coords = [coords._lon(), coords._lat()]
        assert isinstance(coords, Sequence)
        coords = [float(coord) for coord in coords]
        assert len(coords) == 2
        _vroom.Location.__init__(self, coords=coords)
        assert self._has_coordinates()
        assert not self._user_index()

    @property
    def coords(self) -> Tuple[float, float]:
        """Location longitude and latitude."""
        return self._lon(), self._lat()

    def __repr__(self):
        return f"vroom.{self.__class__.__name__}({self.coords})"


class Location(LocationIndex, LocationCoordinates):
    """Location for where a job needs to e done.

    Either as an index referring to a column in the durations matrix, or as
    longitude-latitude coordinates.

    Converts to :cls:`LocationCoordinates` if no `index` is provided, and to
    :cls:`LocationIndex` if not `coords` is provided.

    Attributes:
        index:
            Location index referring to column in the duration
            matrix.
        coords:
            Longitude and latitude coordinate.

    Args:
        index:
            Location index referring to column in the duration
            matrix.
        coords:
            Longitude and latitude coordinate.
        location:
            Other location to make a smart copy of.

    Examples:
        >>> loc = vroom.Location(index=4, coords=(7., 8.))
        >>> loc
        vroom.Location(index=4, coords=(7.0, 8.0))
        >>> loc.index, loc.coords
        (4, (7.0, 8.0))
        >>> vroom.Location(4)
        vroom.LocationIndex(4)
        >>> vroom.Location((7., 8.))
        vroom.LocationCoordinates((7.0, 8.0))

    See also:
        :cls:`vroom.LocationIndex`, :cls:`vroom.LocationCoordinates`

    """

    __init__ = _vroom.Location.__init__

    def __new__(
        cls,
        *args,
        **kwargs,
    ):
        if len(args) + len(kwargs) > 2:
            raise TypeError("too many arguents in input.")
        if args and "index" in kwargs:
            raise TypeError("index reference twice.")

        illegal_kwargs = False
        if len(args) == 2:
            illegal_kwargs = bool(kwargs)
            kwargs["index"], kwargs["coords"] = args
        elif len(args) == 1:
            if isinstance(args[0], _vroom.Location):
                illegal_kwargs = bool(kwargs)
                if args[0]._user_index():
                    kwargs["index"] = args[0]._index()
                if args[0]._has_coordinates():
                    kwargs["coords"] = [args[0]._lon(), args[0]._lat()]

            elif isinstance(args[0], Sequence):
                illegal_kwargs = bool(kwargs)
                kwargs["coords"] = args[0]

            else:
                kwargs["index"] = args[0]

        if illegal_kwargs:
            raise TypeError("too many arguents in input.")

        kwargs = {key: list(value) if isinstance(value, Sequence) else value
                  for key, value in kwargs.items()}

        if cls is Location and len(kwargs) == 1:
            cls = LocationIndex if "index" in kwargs else LocationCoordinates
            instance = _vroom.Location.__new__(cls, **kwargs)
            instance.__init__(**kwargs)
        else:
            instance = _vroom.Location.__new__(cls, **kwargs)
        return instance

    def __repr__(self) -> str:
        args = f"index={self.index}, coords={self.coords}"
        return f"vroom.{self.__class__.__name__}({args})"



"""Time window for when a delivery/pickup/task is possible."""
from __future__ import annotations
from typing import Any, Sequence, Union

import numpy

from . import _vroom

MAX_VAL = numpy.iinfo(numpy.uint32).max


class TimeWindow(_vroom.TimeWindow):
    """Time window for when a delivery/pickup/task is possible.

    Relative values, e.g. `[0, 14400]` for a 4 hours time window starting at
    the beginning of the planning horizon. In that case all times reported in
    output with the arrival key are relative to the start of the planning
    horizon;

    Supprt the following features:

    * No arguments implies no time constraints.
    * Equality operator `==` based on both start and end time are the same.
    * Normal compare operator `<, >, <=, >=` based on start time.
    * Shift `<<, >>` based on non-overlap intervals.
    * Contains `X in Y` based on if number/interval inside other interval.
    * Length `len(X)` give length of interval.
    * Falsy on no constrained interval.

    Attributes:
        start:
            Start point (inclusice) of the time window.
        end:
            End point (inclusive) of the time window.

    Args:
        start:
            Start point (inclusive) of the time window. In
            seconds from the starting time. Assumed `0 < start`.
        end:
            End point (inclusive) of the time window. In
            seconds from the starting time. Assumes `start < end`.

    Examples:
        >>> tw = vroom.TimeWindow(2200, 8800)
        >>> tw
        vroom.TimeWindow(2200, 8800)
        >>> tw.start, tw.end, len(tw)
        (2200, 8800, 6600)
        >>> 1000 in tw, 5000 in tw
        (False, True)

    """

    start: int
    end: int

    def __init__(
        self,
        start: Union[_vroom.TimeWindow, Sequence[int], int] = 0,
        end: int = MAX_VAL,
    ) -> None:
        assert isinstance(end, int)
        if isinstance(start, _vroom.TimeWindow):
            if end != MAX_VAL:
                raise TypeError("Only one arg when input is vroom.TimeWindow.")
            start, end = start.start, start.end
        if isinstance(start, Sequence):
            if end != MAX_VAL:
                raise TypeError("Only one arg when input is a sequence.")
            start, end = start
        _vroom.TimeWindow.__init__(self, start=start, end=end)

    def __bool__(self) -> bool:
        return self.start != 0 or self.end != MAX_VAL

    def __contains__(self, other: Union[_vroom.TimeWindow, int]) -> bool:
        if isinstance(other, int):
            return self._contains(other)
        return self.start < other.start and other.end < self.end

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, _vroom.TimeWindow):
            return self.start == other.start and self.end == other.end
        return NotImplemented

    def __le__(self, other: TimeWindow) -> bool:
        return self.start <= other.start

    def __lshift__(self, other: TimeWindow) -> bool:
        return self.end < other.start

    def __len__(self):
        return self._length

    def __repr__(self):
        args = f"{self.start}, {self.end}" if self else ""
        return f"vroom.{self.__class__.__name__}({args})"

    def __rshift__(self, other: TimeWindow) -> bool:
        return self.start > other.end



from typing import Any, Dict, Union
from pathlib import Path
import io
import json
from contextlib import redirect_stdout

import numpy
import pandas

from .. import _vroom

NA_SUBSTITUTE = 4293967297


class Solution(_vroom.Solution):

    @property
    def routes(self) -> pandas.DataFrame:
        array = numpy.asarray(self._routes_numpy())
        frame = pandas.DataFrame({
            "vehicle_id": array["vehicle_id"],
            "type": pandas.Categorical(
                array["type"].astype("U9"),
                categories=["start", "end", "break", "job", "delivery", "pickup"],
            ),
            "arrival": array["arrival"],
            "duration": array["duration"],
            "setup": array["setup"],
            "service": array["service"],
            "waiting_time": array["waiting_time"],
            "location_index": array["location_index"],
            "longitude": pandas.array(array["longitude"], dtype="Int64"),
            "latitude": pandas.array(array["latitude"], dtype="Int64"),
            "id": pandas.array(array["id"], dtype="Int64"),
            "description": array["description"].astype("U40"),
        })
        for column in ["longitude", "latitude", "id"]:
            if (frame[column] == NA_SUBSTITUTE).all():
                del frame[column]
            else:
                frame.loc[frame[column] == NA_SUBSTITUTE, column] = pandas.NA
        return frame

    def to_dict(self) -> Dict[str, Any]:
        stream = io.StringIO()
        with redirect_stdout(stream):
            self._solution_json()
        return json.loads(stream.getvalue())

    def to_json(self, filepath: Union[str, Path]) -> None:
        with open(filepath, "w") as handler:
            with redirect_stdout(handler):
                self._solution_json()



import vroom


JOB_SINGLE1 = vroom.Job(id=0, location=1, delivery=[4], pickup=[5])
JOB_SINGLE2 = vroom.JobSingle(id=0, location=1, delivery=[4], pickup=[5])
JOB_SINGLE3 = vroom.JobSingle(
    id=1,
    location=2,
    setup=3,
    service=4,
    delivery=[5],
    pickup=[6],
    skills={7},
    priority=8,
    time_windows=[(9, 10)],
    description="11",
)
JOB_SINGLE4 = vroom.Job(id=0, location=1)

JOB_DELIVERY1 = vroom.Job(id=0, location=1, delivery=[4])
JOB_DELIVERY2 = vroom.JobDelivery(id=0, location=1, amount=[4])
JOB_DELIVERY3 = vroom.JobDelivery(
    id=1,
    location=2,
    setup=3,
    service=4,
    amount=[5],
    skills={7},
    priority=8,
    time_windows=[(9, 10)],
    description="11",
)

JOB_PICKUP1 = vroom.Job(id=0, location=1, pickup=[4])
JOB_PICKUP2 = vroom.JobPickup(id=0, location=1, amount=[4])
JOB_PICKUP3 = vroom.JobPickup(
    id=1,
    location=2,
    setup=3,
    service=4,
    amount=[6],
    skills={7},
    priority=8,
    time_windows=[(9, 10)],
    description="11",
)


def test_job_repr():
    assert repr(JOB_SINGLE1) == "vroom.JobSingle(0, 1, delivery=[4], pickup=[5])"
    assert repr(JOB_SINGLE3) == ("vroom.JobSingle(1, 2, setup=3, service=4, delivery=[5], pickup=[6], "
                                 "skills={7}, priority=8, time_windows=[(9, 10)], description='11')")
    assert repr(JOB_SINGLE4) == "vroom.JobSingle(0, 1)"
    assert repr(JOB_DELIVERY1) == "vroom.JobDelivery(0, 1, amount=[4])"
    assert repr(JOB_DELIVERY3) == ("vroom.JobDelivery(1, 2, setup=3, service=4, amount=[5], skills={7}, "
                                   "priority=8, time_windows=[(9, 10)], description='11')")
    assert repr(JOB_PICKUP1) == "vroom.JobPickup(0, 1, amount=[4])"
    assert repr(JOB_PICKUP3) == ("vroom.JobPickup(1, 2, setup=3, service=4, amount=[6], skills={7}, "
                                 "priority=8, time_windows=[(9, 10)], description='11')")


def test_job_subclass():
    # assert isinstance(JOB_SINGLE1, vroom.Job)
    assert isinstance(JOB_SINGLE1, vroom.JobSingle)
    assert not isinstance(JOB_SINGLE1, vroom.JobDelivery)
    assert not isinstance(JOB_SINGLE1, vroom.JobPickup)

    # assert isinstance(JOB_SINGLE2, vroom.Job)
    assert isinstance(JOB_SINGLE2, vroom.JobSingle)
    assert not isinstance(JOB_SINGLE2, vroom.JobDelivery)
    assert not isinstance(JOB_SINGLE2, vroom.JobPickup)

    # assert isinstance(JOB_SINGLE4, vroom.Job)
    assert isinstance(JOB_SINGLE4, vroom.JobSingle)
    assert not isinstance(JOB_SINGLE4, vroom.JobDelivery)
    assert not isinstance(JOB_SINGLE4, vroom.JobPickup)

    # assert isinstance(JOB_DELIVERY1, vroom.Job)
    assert not isinstance(JOB_DELIVERY1, vroom.JobSingle)
    assert isinstance(JOB_DELIVERY1, vroom.JobDelivery)
    assert not isinstance(JOB_DELIVERY1, vroom.JobPickup)

    # assert isinstance(JOB_DELIVERY2, vroom.Job)
    assert not isinstance(JOB_DELIVERY2, vroom.JobSingle)
    assert isinstance(JOB_DELIVERY2, vroom.JobDelivery)
    assert not isinstance(JOB_DELIVERY2, vroom.JobPickup)

    # assert isinstance(JOB_PICKUP1, vroom.Job)
    assert not isinstance(JOB_PICKUP1, vroom.JobSingle)
    assert not isinstance(JOB_PICKUP1, vroom.JobDelivery)
    assert isinstance(JOB_PICKUP1, vroom.JobPickup)

    # assert isinstance(JOB_PICKUP2, vroom.Job)
    assert not isinstance(JOB_PICKUP2, vroom.JobSingle)
    assert not isinstance(JOB_PICKUP2, vroom.JobDelivery)
    assert isinstance(JOB_PICKUP2, vroom.JobPickup)


def test_job_attributes():
    assert JOB_SINGLE3.id == 1
    assert JOB_SINGLE3.location == vroom.Location(2)
    assert JOB_SINGLE3.setup == 3
    assert JOB_SINGLE3.service == 4
    assert not hasattr(JOB_SINGLE3, "amount")
    assert JOB_SINGLE3.delivery == vroom.Amount([5])
    assert JOB_SINGLE3.pickup == vroom.Amount([6])
    assert JOB_SINGLE3.skills == {7}
    assert JOB_SINGLE3.priority == 8
    assert JOB_SINGLE3.time_windows == [vroom.TimeWindow(9, 10)]
    assert JOB_SINGLE3.description == "11"

    assert JOB_SINGLE4.delivery == vroom.Amount([])
    assert JOB_SINGLE4.pickup == vroom.Amount([])

    assert JOB_DELIVERY3.id == 1
    assert JOB_DELIVERY3.location == vroom.Location(2)
    assert JOB_DELIVERY3.setup == 3
    assert JOB_DELIVERY3.service == 4
    assert JOB_DELIVERY3.amount == vroom.Amount([5])
    assert not hasattr(JOB_DELIVERY3, "delivery")
    assert not hasattr(JOB_DELIVERY3, "pickup")
    assert JOB_DELIVERY3.skills == {7}
    assert JOB_DELIVERY3.priority == 8
    assert JOB_DELIVERY3.time_windows == [vroom.TimeWindow(9, 10)]
    assert JOB_DELIVERY3.description == "11"

    assert JOB_PICKUP3.id == 1
    assert JOB_PICKUP3.location == vroom.Location(2)
    assert JOB_PICKUP3.setup == 3
    assert JOB_PICKUP3.service == 4
    assert JOB_PICKUP3.amount == vroom.Amount([6])
    assert not hasattr(JOB_PICKUP3, "delivery")
    assert not hasattr(JOB_PICKUP3, "pickup")
    assert JOB_PICKUP3.skills == {7}
    assert JOB_PICKUP3.priority == 8
    assert JOB_PICKUP3.time_windows == [vroom.TimeWindow(9, 10)]
    assert JOB_PICKUP3.description == "11"



import numpy
import pytest

from vroom import _vroom
import vroom

MAX_VAL = numpy.iinfo(numpy.uint32).max


def test_time_window_init():
    tw = vroom.TimeWindow()
    assert tw.start == 0
    assert tw.end == MAX_VAL
    assert _vroom.TimeWindow().end == MAX_VAL

    with pytest.raises(_vroom.VroomInputException):
        vroom.TimeWindow(1000, 0)

    with pytest.raises(TypeError):
        vroom.TimeWindow(tw, 1000)

    tw = vroom.TimeWindow(1000, 2000)
    assert tw.start == vroom.TimeWindow(tw).start
    assert tw.end == vroom.TimeWindow(tw).end


def test_time_window_compare():
    assert vroom.TimeWindow(50, 150) > vroom.TimeWindow(0, 100)
    assert vroom.TimeWindow(0, 100) < vroom.TimeWindow(50, 150)
    assert vroom.TimeWindow(50, 150) >= vroom.TimeWindow(0, 100)
    assert vroom.TimeWindow(0, 100) <= vroom.TimeWindow(50, 150)
    assert vroom.TimeWindow(0, 100) >= vroom.TimeWindow(0, 100)
    assert vroom.TimeWindow(0, 100) <= vroom.TimeWindow(0, 100)
    assert vroom.TimeWindow(0, 100) != vroom.TimeWindow(0, 200)
    assert vroom.TimeWindow(0, 100) == vroom.TimeWindow(0, 100)

    assert not (vroom.TimeWindow(50, 150) < vroom.TimeWindow(0, 100))
    assert not (vroom.TimeWindow(0, 100) > vroom.TimeWindow(50, 150))
    assert not (vroom.TimeWindow(50, 150) <= vroom.TimeWindow(0, 100))
    assert not (vroom.TimeWindow(0, 100) >= vroom.TimeWindow(50, 150))
    assert not (vroom.TimeWindow(0, 100) != vroom.TimeWindow(0, 100))

def test_time_window_shift():

    assert vroom.TimeWindow(0, 100) << vroom.TimeWindow(200, 300)
    assert vroom.TimeWindow(200, 300) >> vroom.TimeWindow(0, 100)
    assert not (vroom.TimeWindow(0, 100) << vroom.TimeWindow(50, 150))
    assert not (vroom.TimeWindow(0, 100) >> vroom.TimeWindow(50, 150))

    assert not (vroom.TimeWindow(0, 300) == vroom.TimeWindow(100, 200))
    assert not (vroom.TimeWindow(0, 300) << vroom.TimeWindow(100, 200))
    assert not (vroom.TimeWindow(0, 300) >> vroom.TimeWindow(100, 200))


def test_time_window_contains():
    tw = vroom.TimeWindow(100, 200)
    assert not 50 in tw
    assert 150 in tw
    assert not 250 in tw

    assert vroom.TimeWindow(0, 150) not in tw
    assert vroom.TimeWindow(120, 180) in tw
    assert vroom.TimeWindow(120, 280) not in tw


def test_time_window_bool():
    assert vroom.TimeWindow(0, 100)
    assert not vroom.TimeWindow()
